// Store port for native messaging connections
var portDict = {};
var dataResult = "";
var manifest = chrome.runtime.getManifest();

// Listen for messages from content script
chrome.runtime.onMessage.addListener(
    function (request, sender, sendResponse) {
        try {
            if (request.type == "connect") {
                connect(sender.tab);
                sendResponse({ success: true });
            } else if (request.type == "disconnect") {
                disconnect(sender.tab);
                sendResponse({ success: true });
            } else if (request.type == "check") {
                sendResponse({ success: true });
            } else {
                if ( null == portDict[sender.tab.id] )
                {
                    connect(sender.tab);
                }
				sendNativeMessage(request.message, sender.tab);
				document.addEventListener('ICBC-MW-nativeMessage' + sender.tab.id,
					function handleNativeMessage(e) {
						var value = JSON.parse(e.detail);
						var tabId = parseInt(value["TabId"]);
						if (tabId == sender.tab.id) {
							sendMessageByTabId(tabId, e.detail);
						}
						sendResponse({ success: true });
						document.removeEventListener(e.type, handleNativeMessage);
					}, false);
            }
            return true;
        }
        catch (e) {
            sendResponse({ success: false, message: "connect error" })
            return true;
        }
    });
	
function sendMessageByTabId(tabId, message)
{
	if ( tabId )
	{
		chrome.tabs.sendMessage(tabId, message);
	}
	else
	{
		chrome.tabs.query({ active: true, currentWindow: true }, function (tabs){
			var lastTabId = tabs[0].id;
			chrome.tabs.sendMessage(lastTabId, message);
		});
	}
}

function putNativeMessage(message)
{
    if ( "GetVersion" == message.MessageType )
    {
        message.message = manifest.version;
    }
    var msg = JSON.stringify(message);
    var event = new CustomEvent('ICBC-MW-nativeMessage' + message.TabId, { 'detail': msg });
    document.dispatchEvent(event);
    dataResult = "";
}

function onNativeMessage(message) {
	if ( undefined != message )
	{
		try{
			dataResult += message.message;
			if ( "end" == message.status )
			{
				message.message = dataResult;
				putNativeMessage(message);
			}
			else if ( "continue" == message.status )
			{
				// continue
			}
			else if ( "start" == message.status )
			{
				// repost
				portDict[message.TabId].postMessage(message);
			}
		}
		catch(e){
			putNativeMessage(message);
		}
	}
}

// Connect to native app and get the communication port  
function connect(tab) {
    var port = null;

    var hostName = "com.google.chrome.icbc.mw";
    port = chrome.runtime.connectNative(hostName);

    port.onMessage.addListener(onNativeMessage);
    port.onDisconnect.addListener(function onDisconnected() {
        portDict[tab.id].disconnect();
        delete portDict[tab.id];
        });

    var id = tab.id;
    portDict[id] = port;
}

// Disconnect from native app
function disconnect(tab) {
    var id = tab.id;
    portDict[id].disconnect();
    delete portDict[id];
}

// Send message to native app
function sendNativeMessage(message, tab) {
    var id = tab.id;
    try {
        var name = 'TabId';
        message[name] = tab.id;
        var status = 'status';
        message[status] = 'start';

        portDict[id].postMessage(message);
    }
    catch (e) {
        throw e;
    }
}